
// JSonSerial2: C++ Object Serialization in JSON
// See: https://www.telecom-paris.fr/~elc/software/
// This file should be included *after* jsonserial.hpp when serializing std::array
//
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com)
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef jsonserial_array_hpp
#define jsonserial_array_hpp
#include <array>
namespace jsonserial {

template <class T, size_t N>
struct is_std_array<std::array<T,N>> : std::true_type {};

template<class T>
struct JSonArrayImpl<T, typename std::enable_if<is_std_array<T>::value>::type>
: public JSonArray {

  JSonArrayImpl(T& array) : array_(array), index_(0) {}
  
  void add(JSonSerial& js, JSonClass::Creator* cr, JKString& s) override {
    JObjectPtr* objptr{};
    if (index_ >= array_.size()) js.error(JSonError::CantAddToArray);
    else readArrayValue(js, array_[index_++], objptr, cr, s);
  }

  T& array_;
  size_t index_;
};

}
#endif
